/*******************************************************************************
*                                                                              *
*   ThreadComm.cpp :                                                           *
*                                                                              *
*   source code for ThreadComm, thread in charge of communications with mbs270 *
*    . ThreadComm : main function                                              *
*    . tc_init : initialization procedure                                      *
*    . tc_stop : stopping procedure                                            *
*    . tc_processMessage : Message processing function                         *
*    . tc_recvInitData : blocking read of initial data                         *
*                                                                              *
********************************************************************************
*   SUIVI DES MODIFICATIONS                                                    *
*                                                                              *
*   14/09/2006 : cration                                                      *
*   14/01/2007 : modifs protocole communication, machine  tat                *
*                                                                              *
*******************************************************************************/

#include <sys/types.h>
#include <sys/timeb.h>

#include "stdafx.h"
#include "Client_demo.h"

//#define TC_DEF_NO_COMM
//#define TC_DEF_DEBUG

#ifdef TC_DEF_DEBUG
struct _timeb ts, te, tt;
double ts_d, te_d, tt_d, dtr;
#endif

// Color conversion macros
#define CONV_16_240_UC_LUM_TO_0_255_D_LUM(y) (((double) (y) -  16.0) * 1.1384)
#define CONV_16_240_UC_CHR_TO_0_255_D_CHR(c) (((double) (c) - 128.0) * 1.1429)







void tc_traceErr(char* str1, int n, char* str2)
{
	static struct _timeb	t;
	static struct tm		*pDate;

	_ftime(&t);
	pDate = gmtime(&t.time);
	printf("%02d/%02d/%02d %02d:%02d:%02d.%03d %s %4d : %s\n",
			pDate->tm_mday,
			pDate->tm_mon,
			pDate->tm_year-100,
			pDate->tm_hour,
			pDate->tm_min,
			pDate->tm_sec,
			t.millitm, str1, n, str2);
}



void tc_conv_ycrcb_to_bgr(double y, double cr, double cb, unsigned char *dest)
{
		double r, g, b;
	r = y + 1.403 * cr; if (r < 0.0) r = 0.0; if (r > 255.0) r = 255.0;
	g = y - 0.344 * cb - 0.714 * cr; if (g < 0.0) g = 0.0; if (g > 255.0) g = 255.0;
	b = y + 1.770 * cb; if (b < 0.0) b = 0.0; if (b > 255.0) b = 255.0;
	*dest = (unsigned char) b;
	*(dest+1) = (unsigned char) g;
	*(dest+2) = (unsigned char) r;
}


// tc_recvInitData : blocking read of initial data
// return code: 0=OK, !0=error
int tc_recvInitData()
{
	return 0;
}


// tc_recvSocket : blocking read on socket
// return code: 0=OK, !0=error
int tc_recvSocket(char* pBuf, int SzBuf)
{
#ifdef TC_DEF_NO_COMM
	SuspendThread(ghThreadComm);
	return 0;
#else
	int rc, nboct, nb_not_received;
	char *ptr, bDcnx;
	
	rc = 0;
	nb_not_received = SzBuf;
	ptr = pBuf;
	bDcnx = FALSE;

	while (nb_not_received > 0)
	{
		nboct = recv(gSocketComm, ptr, nb_not_received, 0);
		if (nboct == SOCKET_ERROR)
		{
			// Erreur
			tc_traceErr("tc_recvSocket", __LINE__, "Error recv");
			printf("EC = %d\n",WSAGetLastError());
			rc = -1;
			break;
		}
		else if (nboct == 0)
		{
			// mbs270 is disconnected
			tc_traceErr("tc_recvSocket", __LINE__, "mbs270 disconnected");
			rc = -1;
			break;
		}
		else
		{
			nb_not_received = nb_not_received - nboct;
			ptr = ptr + nboct;
		}
	}

	return rc;
#endif
}


// sendSocket : blocking write to mbs270
// return code: none, if write fail, application is stopped
void sendSocket(char* pBuf, int szBuf)
{
	printf("sendSocket %d, szBuf = %d\n", *((int*)pBuf), szBuf);
#ifdef TC_DEF_NO_COMM
	return;
#else
	int rc, nboct, nb_not_sent;
	char* ptr;

	rc = 1;
	nb_not_sent = szBuf;
	ptr = pBuf;

	while (nb_not_sent > 0)
	{
		nboct = send(gSocketComm, ptr, nb_not_sent, 0);
		if (nboct == SOCKET_ERROR)
		{
			tc_traceErr("sendSocket", __LINE__, "Error send");
			printf("EC = %d\n",WSAGetLastError());
			rc = 0;
			break;	// stop sending
		}
		else
		{
			nb_not_sent = nb_not_sent - nboct;
			ptr = ptr + nboct;
		}
	}

	if (rc == 0)
	{
		// stop client
		PostQuitMessage(0);
	}
#endif
}


// tc_init : initialization procedure
// return code: 0=OK, !0=error
int tc_init()
{

	int rc = 0;
#ifdef TC_DEF_NO_COMM
	if (1) {
		if (1) {
#else
	// TCP/IP socket creation
	gSocketComm = socket(AF_INET, SOCK_STREAM, 0);
	if (gSocketComm == INVALID_SOCKET)
	{
		tc_traceErr("tc_init", __LINE__, "Error socket");
		printf(" EC = %d", WSAGetLastError());
		rc = -1;
	}
	else
	{
		SOCKADDR_IN sin;
		sin.sin_addr.s_addr = inet_addr(gIPadr);
		sin.sin_family	    = AF_INET;
		sin.sin_port	    = htons(MD_SOCKET_PORT);

		// Connexion attempt
		if (connect(gSocketComm, (SOCKADDR *)&sin, sizeof(sin)) == 0)
		{
#endif
			tc_traceErr("tc_init", __LINE__, "Connexion avec mbs270 tablie");
		}
		else
		{
			tc_traceErr("tc_init", __LINE__, "Error connect");
			printf("EC = %d\n", WSAGetLastError());
			rc = -1;
		}
	}

	gInit_rc = rc;
	return rc;
}

		
// tc_processMessage : message processing function
// return code: 0=OK, !0=error
int tc_processMessage(md_msghdr_t *pHdr)
{
	static unsigned char image[K_IMGMAXW*K_IMGMAXH*3], *puc;
	unsigned int compute_time_us;
	int ii;
	int rc = 0;
	char str2display[100];

	switch (pHdr->type)
	{
	case MD_MSG_INITDATA :
		if (tc_recvInitData() != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvInitData");
			rc = -1;
		}
		break;

	case MD_MSG_STRING :
#ifdef TC_DEF_DEBUG
printf("before tc_recvSocket string, len = %d\n", pHdr->bodylen);
#endif
		if (tc_recvSocket(str2display, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
			SetDlgItemText(ghClient_demoDlg, IDC_STATIC_MESSAGE, str2display);
		break;

	case MD_MSG_IMG_Y8 :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket Y8 image, size = %d\n", tt_d, pHdr->bodylen);
#endif
		if (tc_recvSocket((char*)&image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (gMode == K_MODE_Y8)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				SetDlgItemText(ghClient_demoDlg, IDC_STATIC_MESSAGE, gtMode[gMode].active_str);

				// Convert Y8 image to RGB24 grey level image
				puc = gImage;
				int y;
				for (ii=0; ii<pHdr->bodylen; ii++)
				{
					y = ((int) image[ii] - 16) * 255;
					y = y / 224; if (y < 0) y = 0; if (y > 255) y = 255;
					*puc = y; puc++;
					*puc = y; puc++;
					*puc = y; puc++;
				}
				// refresh image display
				if (ghClient_demoDlg != NULL) {
					InvalidateRect(ghClient_demoDlg, NULL, FALSE);
					UpdateWindow(ghClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_YCRCB422_PK :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket YCrCb422 image, size = %d\n", tt_d, pHdr->bodylen);
#endif
		if (tc_recvSocket((char*)&image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (gMode == K_MODE_YCRCB422_PK)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				SetDlgItemText(ghClient_demoDlg, IDC_STATIC_MESSAGE, gtMode[gMode].active_str);

				// Convert YCrCb422 image to RGB24 image
				// Y, Cr and Cb components are packed as Cbn Yn Crn Yn+1
				puc = gImage;
				double y, cr, cb;
				for (ii=0; ii<pHdr->bodylen/4; ii++)
				{
					y  = CONV_16_240_UC_LUM_TO_0_255_D_LUM(image[ii*4+1]);
					cr = CONV_16_240_UC_CHR_TO_0_255_D_CHR(image[ii*4+2]);
					cb = CONV_16_240_UC_CHR_TO_0_255_D_CHR(image[ii*4  ]);
					tc_conv_ycrcb_to_bgr(y, cr, cb, puc); puc += 3;

					y  = CONV_16_240_UC_LUM_TO_0_255_D_LUM(image[ii*4+3]);
					tc_conv_ycrcb_to_bgr(y, cr, cb, puc); puc += 3;
				}
				// refresh image display
				if (ghClient_demoDlg != NULL) {
					InvalidateRect(ghClient_demoDlg, NULL, FALSE);
					UpdateWindow(ghClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_YCRCB422_PL :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket YCrCb422 image, size = %d\n", tt_d, pHdr->bodylen);
#endif
		if (tc_recvSocket((char*)&image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (gMode == K_MODE_YCRCB422_PL)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				SetDlgItemText(ghClient_demoDlg, IDC_STATIC_MESSAGE, gtMode[gMode].active_str);

				// Convert YCrCb422 image to RGB24 image
				// Y, Cr and Cb components are in 3 separate planes:
				//   Y plane starts at image and is defined for every pixels
				//   Cr and Cb are defined for half pixel only; Cb plane starts at
				//   image + pHdr->bodylen/2 and Cr at image + pHdr->bodylen * 3/4.
				puc = gImage;
				unsigned char *py  = image;
				unsigned char *pcb = py  + pHdr->bodylen / 2;
				unsigned char *pcr = pcb + pHdr->bodylen / 4;
				double y, cr, cb;
				for (ii=0; ii<pHdr->bodylen/4; ii++)
				{
					y  = CONV_16_240_UC_LUM_TO_0_255_D_LUM(*py );
					cr = CONV_16_240_UC_CHR_TO_0_255_D_CHR(*pcr);
					cb = CONV_16_240_UC_CHR_TO_0_255_D_CHR(*pcb);
					tc_conv_ycrcb_to_bgr(y, cr, cb, puc); puc += 3;
					py++;
					y  = CONV_16_240_UC_LUM_TO_0_255_D_LUM(*py );
					tc_conv_ycrcb_to_bgr(y, cr, cb, puc); puc += 3;
					py++; pcr++; pcb++;
				}
				// refresh image display
				if (ghClient_demoDlg != NULL) {
					InvalidateRect(ghClient_demoDlg, NULL, FALSE);
					UpdateWindow(ghClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_RAW_RGB_PL :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket image\n", tt_d);
#endif
		if (tc_recvSocket((char*)&image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (gMode == K_MODE_RAW_RGB_PL)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				SetDlgItemText(ghClient_demoDlg, IDC_STATIC_MESSAGE, gtMode[gMode].active_str);
printf("r");
				// Convert raw RGB image to RGB24 image
				// R, G and B components are in 3 separate planes:
				//   G plane starts at image and is defined for every pixels
				//   R and B are defined for half pixel only; B plane starts at
				//   image + pHdr->bodylen/2 and R at image + pHdr->bodylen * 3/4.
				puc = gImage;
				unsigned char *pg = image;
				unsigned char *pb = pg + pHdr->bodylen / 2;
				unsigned char *pr = pb + pHdr->bodylen / 4;
				for (ii=0; ii<pHdr->bodylen/4; ii++)
				{
					*puc = *pb; puc++; // Bn
					*puc = *pg; puc++; // Gn
					*puc = *pr; puc++; // Rn
					pg++;
					*puc = *pb; puc++; // Bn+1
					*puc = *pg; puc++; // Gn+1
					*puc = *pr; puc++; // Rn+1
					pg++; pr++; pb++;
				}

				// refresh image display
				if (ghClient_demoDlg != NULL) {
					InvalidateRect(ghClient_demoDlg, NULL, FALSE);
					UpdateWindow(ghClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_GRAD_H :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket horizontal gradient image\n", tt_d);
#endif
		if (tc_recvSocket((char*)&image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (gMode == K_MODE_GRAD_H)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				/* computation time is after gradient data */
				compute_time_us = *((int*)(image + pHdr->bodylen - sizeof(int)));
				sprintf(str2display, gtMode[gMode].active_str, compute_time_us);
				SetDlgItemText(ghClient_demoDlg, IDC_STATIC_MESSAGE, str2display);

				// Convert 8bits/pixel image to RGB24 grey level image
				puc = gImage;
				for (ii=0; ii<pHdr->bodylen - sizeof(int); ii++)
				{
					*puc = image[ii]; puc++;
					*puc = image[ii]; puc++;
					*puc = image[ii]; puc++;
				}
				/* raffraichissement affichage image */
				if (ghClient_demoDlg != NULL) {
					InvalidateRect(ghClient_demoDlg, NULL, FALSE);
					UpdateWindow(ghClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_GRAD_V :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket vertical gradient image\n", tt_d);
#endif
		if (tc_recvSocket((char*)&image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (gMode == K_MODE_GRAD_V)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				/* computation time is after gradient data */
				compute_time_us = *((int*)(image + pHdr->bodylen - sizeof(int)));
				sprintf(str2display, gtMode[gMode].active_str, compute_time_us);
				SetDlgItemText(ghClient_demoDlg, IDC_STATIC_MESSAGE, str2display);

				// Convert 8bits/pixel image to RGB24 grey level image
				puc = gImage;
				for (ii=0; ii<pHdr->bodylen - sizeof(int); ii++)
				{
					*puc = image[ii]; puc++;
					*puc = image[ii]; puc++;
					*puc = image[ii]; puc++;
				}
				/* raffraichissement affichage image */
				if (ghClient_demoDlg != NULL) {
					InvalidateRect(ghClient_demoDlg, NULL, FALSE);
					UpdateWindow(ghClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_EDGE :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket vertical gradient image\n", tt_d);
#endif
		if (tc_recvSocket((char*)&image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (gMode == K_MODE_EDGE)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				/* computation time is after gradient data */
				compute_time_us = *((int*)(image + pHdr->bodylen - sizeof(int)));
				sprintf(str2display, gtMode[gMode].active_str, compute_time_us);
				SetDlgItemText(ghClient_demoDlg, IDC_STATIC_MESSAGE, str2display);

				// Compute gradient norm and display in RGB24
				puc = gImage;
				unsigned char norm;
				for (ii=0; ii<pHdr->bodylen - sizeof(int); ii+=2)
				{
					norm = (unsigned char) sqrt((double) *((unsigned short*)(image+ii)));
					*puc = norm; puc++;
					*puc = norm; puc++;
					*puc = norm; puc++;
				}
				/* raffraichissement affichage image */
				if (ghClient_demoDlg != NULL) {
					InvalidateRect(ghClient_demoDlg, NULL, FALSE);
					UpdateWindow(ghClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_REG_VAL :
#ifdef TC_DEF_DEBUG
printf("before tc_recvSocket regval, len = %d\n", pHdr->bodylen);
#endif
		int reg_val;
		if (tc_recvSocket((char*)&reg_val, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			char val_str[10];
			if ((reg_val < 0) || (reg_val > 255))
			{
				tc_traceErr("tc_processMessage", __LINE__, "Invalid register value");
				sprintf(val_str, "???");
			}
			else
			{
				sprintf(val_str, "0x%02x", reg_val);
			}
			SetDlgItemText(ghRegisterSettingDlg, IDC_EDIT_REGISTER_VALUE, val_str);
		}
		break;

	default :
		AfxMessageBox("Unknown message header");

	}	// end switch (pHdr->type)

	return rc;
}


// tc_stop : stopping procedure
void tc_stop()
{
	// close socket
	printf("tc_stop: close socket\n");
	if (gSocketComm != INVALID_SOCKET) closesocket(gSocketComm);
	gSocketComm = INVALID_SOCKET;

	// alert Dialog on communication problem
	PostQuitMessage(0);
}


// ThreadComm thread entry point
DWORD WINAPI ThreadComm( LPVOID )
{
	char bStop;
	md_msghdr_t	MsgHdr;
 
	printf("   >>>   ThreadComm starts...\n");

	bStop = FALSE;
	if (tc_init() != 0)
	{
		tc_traceErr("ThreadComm", __LINE__, "Error initializing ThreadComm");
		bStop = TRUE;
	}

	while (bStop == FALSE)
	{
		// blocking read of message header
#ifdef TC_DEF_DEBUG
_ftime( &ts ); ts_d = (double) (ts.time) + (double) (ts.millitm) / 1000.0;
printf("%f before tc_recvSocket header\n", ts_d);
#endif
		if (tc_recvSocket((char*)&MsgHdr, sizeof(MsgHdr)) != 0)
		{
			tc_traceErr("ThreadComm", __LINE__, "Error tc_recvSocket");
			bStop = TRUE;
		}
		else
		{
#ifdef TC_DEF_DEBUG
tc_traceErr("ThreadComm", __LINE__, "Rception en-tte");
printf("Type = %d, len = %d\n", MsgHdr.type, MsgHdr.bodylen);
#endif
			/* Message processing */
			if (tc_processMessage(&MsgHdr) != 0)
			{
				tc_traceErr("ThreadComm", __LINE__, "Echec appel tc_processMessage");
				bStop = TRUE;
			}
#ifdef TC_DEF_DEBUG
			else
			{
				_ftime( &te ); te_d = (double) (te.time) + (double) (te.millitm) / 1000.0;
				dtr = te_d - ts_d;
				printf("End receive %d      Duration = %f\n", MsgHdr.type, dtr);
			}
#endif
		}

	}	// end while (bStop == FALSE)

	tc_stop();
	return 0;
}
