#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/miscdevice.h>
#include <pthread.h>
#include <asm/errno.h>

#define NB_READ_T1 60
#define NB_READ_T2 40
#define NB_READ_T3 60

#define PERIOD_T1 1000000
#define PERIOD_T2 2000000
#define PERIOD_T3  500000

#define ADC_DEV_NAME "/dev/ucb1x00-adc"

int adc_dev;

void *thread1(void *arg)
{
  int ii, cc;
  struct misc_adc adc_data;

  printf("Thread1 starting...\n");

  adc_data.channel_count = 4;
  adc_data.channel[0].number = 0;
  adc_data.channel[1].number = 1;
  adc_data.channel[2].number = 2;
  adc_data.channel[3].number = 3;
  for (ii=0; ii<NB_READ_T1; ii++) {
    usleep(PERIOD_T1);
    if (ioctl(adc_dev, MISCIOCADC, (void*) &adc_data) < 0) {
      perror("ioctl ADC");
      break;
    }
    else {
      printf("T1 % 4d: ", ii);
      for (cc=0; cc<adc_data.channel_count; cc++)
	printf("% 4u  ", adc_data.channel[cc].value);
      printf("\n");
    }
  }

  printf("Thread1 ending.\n");
  pthread_exit(0);
}

void *thread2(void *arg)
{
  int ii, cc;
  struct misc_adc adc_data;

  printf("Thread2 starting...\n");

  adc_data.channel_count = 2;
  adc_data.channel[0].number = 0;
  adc_data.channel[1].number = 1;
  for (ii=0; ii<NB_READ_T2; ii++) {
    usleep(PERIOD_T2);
    if (ioctl(adc_dev, MISCIOCADC, (void*) &adc_data) < 0) {
      perror("ioctl ADC");
      break;
    }
    else {
      printf("T2 % 4d: ", ii);
      for (cc=0; cc<adc_data.channel_count; cc++)
	printf("% 4u  ", adc_data.channel[cc].value);
      printf("\n");
    }
  }

  printf("Thread2 ending.\n");
  pthread_exit(0);
}

void *thread3(void *arg)
{
  int ii, cc;
  struct misc_adc adc_data;

  printf("Thread3 starting...\n");

  adc_data.channel_count = 2;
  adc_data.channel[0].number = 2;
  adc_data.channel[1].number = 3;
  for (ii=0; ii<NB_READ_T3; ii++) {
    usleep(PERIOD_T3);
    if (ioctl(adc_dev, MISCIOCADC, (void*) &adc_data) < 0) {
      perror("ioctl ADC");
      break;
    }
    else {
      printf("T3 % 4d: ", ii);
      for (cc=0; cc<adc_data.channel_count; cc++)
	printf("% 4u  ", adc_data.channel[cc].value);
      printf("\n");
    }
  }

  printf("Thread3 ending.\n");
  pthread_exit(0);
}

int main(int argc, char *argv[])
{
  int ret;
  pthread_t thread1_id, thread2_id, thread3_id;

  printf("\n\n   ***   test_adc_mt   ***\n\n");
  printf("Multithreaded test for UCB1400 ADC driver\n");
  printf("  Thread1 acquires %d times channels AD0 through AD3 with %dms periodicity,\n", NB_READ_T1, PERIOD_T1/1000);
  printf("  Thread2 acquires %d times channels AD0 and AD1 with %dms periodicity,\n",     NB_READ_T2, PERIOD_T2/1000);
  printf("  Thread3 acquires %d times channels AD2 and AD3 with %dms periodicity.\n",     NB_READ_T3, PERIOD_T3/1000);
  
  adc_dev = open(ADC_DEV_NAME, O_RDWR);
  if (adc_dev < 0) {
    printf("%s ", ADC_DEV_NAME);
    perror("open failed");
    exit(1);
  }
  printf("%s open succeeded.\n", ADC_DEV_NAME);
  
  if (pthread_create(&thread1_id, NULL, thread1, NULL)) {
    perror("pthread_create T1");
  }
  
  if (pthread_create(&thread2_id, NULL, thread2, NULL)) {
    perror("pthread_create T2");
  }
  
  if (pthread_create(&thread3_id, NULL, thread3, NULL)) {
    perror("pthread_create T3");
  }

  ret = pthread_join(thread1_id, NULL);
  if (ret && ret!=ESRCH) {
    perror("pthread_join T1");
  }

  ret = pthread_join(thread2_id, NULL);
  if (ret && ret!=ESRCH) {
    perror("pthread_join T2");
  }

  ret = pthread_join(thread3_id, NULL);
  if (ret && ret!=ESRCH) {
    perror("pthread_join T3");
  }

  close(adc_dev);
  printf("End of test.\n");
}
