#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/pxa_gpio.h>

#define GPIO_DEV_NAME "/dev/pxa-gpio"

int main(int argc, char *argv[])
{
  int gpio_dev, ii;
  int toggle_delay, duration;
  struct pxa_gpio_add gpio_add;
  struct timeval tvs, tv;

  printf("\n\n   ***   test_gpio   ***\n\n");
  printf("Basic test for PXA GPIO driver\n\n");
  usleep(100000); /* for proper printing */

  if (argc != 4) goto call_error;
  gpio_add.gpio = (unsigned int) atoi(argv[1]);
  toggle_delay = (unsigned int) atoi(argv[2]);
  duration = atoi(argv[3]);
  
  gpio_dev = open(GPIO_DEV_NAME, O_RDWR);
  if (gpio_dev < 0) {
    printf("%s ", GPIO_DEV_NAME);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.\n", GPIO_DEV_NAME);

  /* output test */
  gpio_add.input = 0;
  gpio_add.init_state = 1;
  if (ioctl(gpio_dev, PXAIOCGPIOADD, (void*) &gpio_add)) {
    perror("ioctl PXAIOCGPIOADD: ");
  }
  else {
    printf("Toggle of GPIO %d every %d microseconds during %d seconds.\n",
	   gpio_add.gpio, toggle_delay, duration);
    gettimeofday(&tvs, NULL);
    while (1) {
      gettimeofday(&tv, NULL);
      if (tv.tv_sec > tvs.tv_sec + duration) break;
      usleep(toggle_delay);
      if (ioctl(gpio_dev, PXAIOCGPIOSET, (void*) &gpio_add.gpio))
	perror("ioctl PXAIOCGPIOSET: ");
      usleep(toggle_delay);
      if (ioctl(gpio_dev, PXAIOCGPIOCLR, (void*) &gpio_add.gpio))
	perror("ioctl PXAIOCGPIOCLR: ");
    }
  }

  /* end of output test */
  if (ioctl(gpio_dev, PXAIOCGPIOREM, (void*) &gpio_add.gpio)) {
    perror("ioctl PXAIOCGPIOREM: ");
    goto close_gpio_device;
  }


  /* input test */
  unsigned int gpio_state;
  struct pxa_gpio_get gpio_get;
  gpio_add.input = 1;
  if (ioctl(gpio_dev, PXAIOCGPIOADD, (void*) &gpio_add)) {
    perror("ioctl PXAIOCGPIOADD: ");
  }
  else {
    printf("GPIO %u initial input state = ", gpio_add.gpio);
    gpio_state = gpio_add.init_state;
    gpio_get.gpio = gpio_add.gpio;
    if (gpio_state) printf("1\n");
    else printf("0\n");
    printf("Change GPIO %d level during %d seconds and check results.\n",
	   gpio_add.gpio, duration);
    gettimeofday(&tvs, NULL);
    while (1) {
      gettimeofday(&tv, NULL);
      if (tv.tv_sec > tvs.tv_sec + duration) break;
      usleep(100);
      if (ioctl(gpio_dev, PXAIOCGPIOGET, (void*) &gpio_get))
	perror("ioctl PXAIOCGPIOGET: ");
      else {
	if ((gpio_get.state) && !(gpio_state)) {
	  gpio_state = gpio_get.state;
	  printf("GPIO %d set.\n", gpio_add.gpio);
	}
	else if (!(gpio_get.state) && (gpio_state)) {
	  gpio_state = gpio_get.state;
	  printf("GPIO %d cleared.\n", gpio_add.gpio);
	}
      }
    }
  }

  /* end of input test */
  if (ioctl(gpio_dev, PXAIOCGPIOREM, (void*) &gpio_add.gpio)) {
    perror("ioctl PXAIOCGPIOREM: ");
    goto close_gpio_device;
  }

 close_gpio_device:
  usleep(100000); // for proper printing
  close(gpio_dev);
  exit(0);
    
 call_error:
  printf("Error. Call syntax is: test_gpio GPIO_number toggle_delay duration\n");
  printf("   toggle_delay expressed in microseconds\n");
  printf("   duration is the test duration in seconds\n");
  exit(1);
}
