#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>

#define I2C_DEV_NAME "/dev/i2c-%d"

int main(int argc, char *argv[])
{
  int i2c_dev, port;
  unsigned int addr, reg;
  char dev_name[sizeof(I2C_DEV_NAME)+4];

  printf("\n\n   ***   test_i2c   ***\n\n");
  printf("Basic test for I2C bus from user space\n\n");
  usleep(100000); /* for proper printing */

  if (argc != 4) goto call_error;
  port = (unsigned int) atoi(argv[1]);
  if (port < 0 || port > 255) goto call_error;
  sscanf(argv[2], "%x", &addr);
  if (addr < 0 || addr > 0xff) goto call_error;
  sscanf(argv[3], "%x", &reg);
  if (addr < 0 || addr > 0xff) goto call_error;
  
  sprintf(dev_name, I2C_DEV_NAME, port);
  i2c_dev = open(dev_name, O_RDWR);
  if (i2c_dev < 0) {
    printf("%s ", dev_name);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.\n", dev_name);

  /* select device */
  if (ioctl(i2c_dev, I2C_SLAVE, (int) addr) < 0) {
    printf("select device 0x%x ", addr);
    perror("ioctl I2C_SLAVE");
    goto close_i2c_device;
  }

  printf("Declared device at address 0x%x\n", addr);

/*   struct i2c_smbus_ioctl_data io_data; */
/*   union i2c_smbus_data data; */
/*   io_data.read_write = I2C_SMBUS_READ; */
/*   io_data.command = I2C_SMBUS_BYTE_DATA; */
/*   io_data.size = I2C_SMBUS_BYTE; */
/*   io_data.data = &data; */
  
/*   if (ioctl(i2c_dev, I2C_SMBUS, &io_data) < 0) { */
/*     printf("select device 0x%x ", addr); */
/*     perror("ioctl I2C_SLAVE"); */
/*     goto close_i2c_device; */
/*   } */

  char buf[2];
  buf[0] = reg;
  if (write(i2c_dev, buf, 1) != 1)
    perror("write");
  else {
    if (read(i2c_dev, buf, 2) != 2) perror("read");
    else printf("Register 0x%x = 0x%x\n", reg, buf[1]);
  }

 close_i2c_device:
  usleep(100000); // for proper printing
  close(i2c_dev);
  exit(0);
    
 call_error:
  printf("Error. Call syntax is: test_i2c addr reg\n");
  printf("   i2c port (value between 0 and 255 after /dev/i2c-)\n");
  printf("   i2c device address (between 0x00 and 0xFF)\n");
  printf("   reg register address (between 0x00 and 0xFF)\n");
  exit(1);
}
