#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#define CONFIG_PXA27x
#include <linux/pxa_pwm.h>
#undef CONFIG_PXA27x

#define PWM_DEV_NAME "/dev/pxa-pwm"

int main(int argc, char *argv[])
{
  int pwm_dev, ii;
  unsigned int duration;
  struct pxa_pwm_add pwm_add;

  printf("\n\n   ***   test_pwm   ***\n\n");
  printf("Basic test for PXA PWM driver\n\n");
  usleep(100000); /* for proper printing */

  if (argc != 5) goto call_error;
  pwm_add.nr = (unsigned int) atoi(argv[1]);
  if (pwm_add.nr >= NR_PXA_PWM) goto call_error;
  pwm_add.f_hz = (unsigned int) atoi(argv[2]);
  pwm_add.duty = atoi(argv[3]);
  duration = atoi(argv[4]);
  
  pwm_dev = open(PWM_DEV_NAME, O_RDWR);
  if (pwm_dev < 0) {
    printf("%s\n", PWM_DEV_NAME);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.\n", PWM_DEV_NAME);

  if (ioctl(pwm_dev, PXAIOCPWMADD, (void*) &pwm_add)) {
    perror("ioctl PXAIOCPWMADD: ");
  }
  else {
    printf("PWM%d outputs signal at %uHz with duty cycle of %u during %u seconds.\n",
	  pwm_add.nr, pwm_add.f_hz, pwm_add.duty, duration);
    sleep(duration);
    if (ioctl(pwm_dev, PXAIOCPWMREM, (void*) &pwm_add.nr)) {
      perror("ioctl PXAIOCPWMREM: ");
    }
  }

  close(pwm_dev);
  exit(0);
    
 call_error:
  printf("Error. Call syntax is: test_pwm PWM_number frequency duty_cycle duration\n");
  printf("   PWM_number is the PWM output number between 0 and %d\n",
	 NR_PXA_PWM-1);
  printf("   frequency is the PWM frequency in Hertz\n");
  printf("   duty_cycle is the PWM duty cycle between 0 (0%%) and 1024 (100%%)\n");

  printf("   duration is the test duration in seconds\n");
  exit(1);
}
