#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/pxa_servo.h>

#define SERVO_DEV_NAME "/dev/pxa-servo"

int main(int argc, char *argv[])
{
  int servo_dev, ii;
  int duration;
  struct pxa_servo_add srv_add;
  struct pxa_servo_rem srv_rem;

  printf("\n\n   ***   test_servo   ***\n\n");
  printf("Basic test for PXA RC servomotor driver\n\n");
  usleep(100000); /* for proper printing */

  if (argc != 4) goto call_error;
  srv_add.gpio = (unsigned int) atoi(argv[1]);
  srv_add.delay = (unsigned int) atoi(argv[2]);
  duration = atoi(argv[3]);
  if (duration <= 0) goto call_error;
  
  servo_dev = open(SERVO_DEV_NAME, O_RDWR);
  if (servo_dev < 0) {
    printf("%s ", SERVO_DEV_NAME);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.\n", SERVO_DEV_NAME);

  if (ioctl(servo_dev, PXAIOCSRVADD, (void*) &srv_add)) {
    perror("ioctl PXAIOCSRVADD: ");
  }
  else {
    printf("%d microseconds pulse on GPIO %d during %d seconds.\n",
	   srv_add.delay, srv_add.gpio, duration);
    sleep(duration);
    srv_rem.id = srv_add.id;
    if (ioctl(servo_dev, PXAIOCSRVREM, (void*) &srv_rem))
      perror("ioctl PXAIOCSRVREM: ");
  }

  usleep(100000); /* for proper printing */
  close(servo_dev);
  exit(0);
    
  call_error:
  printf("Error. Call syntax is: test_servo GPIO_number pulse_width duration\n");
  printf("   pulse_width expressed in microseconds\n");
  printf("   duration is the test duration in seconds\n");
  exit(1);
}
