#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/pxa_servo.h>

#define SERVO_DEV_NAME "/dev/pxa-servo"
#define INIT_PULSE_US  1500
#define MIN_PULSE_US   1000
#define MAX_PULSE_US   2000

int main(int argc, char *argv[])
{
  int servo_dev, ii, inc;
  int duration;
  struct pxa_servo_add srv_add;
  struct pxa_servo_chg srv_chg;
  struct pxa_servo_val srv_val;

  printf("\n\n   ***   test_servo2   ***\n\n");
  printf("Basic test for PXA RC servomotor driver\n\n");
  usleep(100000); /* for proper printing */

  if (argc != 5) goto call_error;
  srv_add.gpio = (unsigned int) atoi(argv[1]);
  srv_add.delay = (unsigned int) atoi(argv[2]);
  inc = atoi(argv[3]);
  duration = atoi(argv[4]);
  if (duration <= 0) goto call_error;
  
  servo_dev = open(SERVO_DEV_NAME, O_RDWR);
  if (servo_dev < 0) {
    printf("%s ", SERVO_DEV_NAME);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.\n", SERVO_DEV_NAME);

  if (ioctl(servo_dev, PXAIOCSRVADD, (void*) &srv_add)) {
    perror("ioctl PXAIOCSRVADD: ");
  }
  else {
    printf("Variable pulse on GPIO %d during %d seconds.\n",
	   srv_add.gpio, duration);

    srv_chg.nb = 1;
    srv_chg.val = &srv_val;
    srv_val.id = srv_add.id;
    srv_val.delay = srv_add.delay;
    for (ii=0; ii<duration; ii++) {
      sleep(1);
      srv_val.delay += inc;
      if (srv_val.delay > MAX_PULSE_US) {
	srv_val.delay = MAX_PULSE_US;
	inc = -inc;
      }
      if (srv_val.delay < MIN_PULSE_US) {
	srv_val.delay = MIN_PULSE_US;
	inc = -inc;
      }
      if (ioctl(servo_dev, PXAIOCSRVCHG, (void*) &srv_chg))
	perror("ioctl PXAIOCSRVCHG: ");
      else printf("Pulse width = %d microseconds\n", srv_val.delay);
    }
  }

  usleep(100000); /* for proper printing */
  close(servo_dev);
  exit(0);
    
  call_error:
  printf("Error. Call syntax is: test_servo2 GPIO_number pulse_width \
increment duration\n");
  printf("   pulse_width expressed in microseconds\n");
  printf("   increment of pulse duration in microseconds\n");
  printf("   duration is the test duration in seconds\n");
  exit(1);
}
