#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/pxa_servo.h>

#define SERVO_DEV_NAME "/dev/pxa-servo"
#define TIME_STEP_US   100000
#define PULSE_INCR         50
#define MIN_PULSE_US      600
#define MAX_PULSE_US     2400

int main(int argc, char *argv[])
{
  int servo_dev, duration, ii, nb_servos;
  struct pxa_servo_add srv_add[NR_PXA_SERVO];
  struct pxa_servo_chg srv_chg;
  struct pxa_servo_val srv_val[NR_PXA_SERVO];
  int delay_incr[NR_PXA_SERVO];
  struct timeval tvs, tv;

  printf("\n\n   ***   test_servo_n   ***\n\n");
  printf("Multiple servo control with PXA RC servomotor driver\n\n");

  /* check arguments */
  if ((argc < 3) || (argc > NR_PXA_SERVO+2)) 
    goto call_error;
  duration = atoi(argv[1]);
  if (duration <= 0)
    goto call_error;

  nb_servos = argc - 2;
  usleep(100000);
  
  servo_dev = open(SERVO_DEV_NAME, O_RDWR);
  if (servo_dev < 0) {
    printf("%s ", SERVO_DEV_NAME);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.\n", SERVO_DEV_NAME);

  for (ii=0; ii<nb_servos; ii++) {
    srv_add[ii].gpio  = atoi(argv[ii+2]);
    srv_add[ii].delay = 1500; /* neutral position */
    if (ioctl(servo_dev, PXAIOCSRVADD, (void*) &srv_add[ii]))
      perror("ioctl PXAIOCSRVADD: ");
    delay_incr[ii] = PULSE_INCR;
    srv_val[ii].id = srv_add[ii].id;
    srv_val[ii].delay = srv_add[ii].delay;
  }
  srv_chg.nb = nb_servos;
  srv_chg.val = srv_val;

  gettimeofday(&tvs, NULL);

  while (1) {
    gettimeofday(&tv, NULL);
    if (tv.tv_sec > tvs.tv_sec + duration) break;
    usleep(TIME_STEP_US);
    for (ii=0; ii<nb_servos; ii++) {
      srv_val[ii].delay += delay_incr[ii];
      if (srv_val[ii].delay > MAX_PULSE_US) {
	srv_val[ii].delay = MAX_PULSE_US;
	delay_incr[ii] = -delay_incr[ii];
      }
      if (srv_val[ii].delay < MIN_PULSE_US) {
	srv_val[ii].delay = MIN_PULSE_US;
	delay_incr[ii] = -delay_incr[ii];
      }
    }
    if (ioctl(servo_dev, PXAIOCSRVCHG, (void*) &srv_chg))
	perror("ioctl PXAIOCSRVCHG: ");
  }

  close(servo_dev);
  exit(0);
    
  call_error:
  printf("Error. Call syntax is: test_servo_n duration a b c ...\n\
            where duration is the test duration in seconds and \n\
            a, b,c ... are up to %d free GPIO numbers.\n",
	 NR_PXA_SERVO);
  exit(1);
}
