#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/pxa_timer.h>

#define TIMER_DEV_NAME     "/dev/pxa-timer"

int main(int argc, char *argv[])
{
  int timer_dev, ii, freq_step;
  unsigned int duration, freq_min, freq_max;
  unsigned long step_delay_us;
  struct pxa_timer_add_chout timer_add;

  printf("\n\n   ***   test_timer2   ***\n\n");
  printf("Multichannel test for PXA TIMER driver\n\n");
  usleep(100000); /* for proper printing */

  if (argc != 6) goto call_error;
  freq_min  = (unsigned int) atoi(argv[1]);
  freq_max  = (unsigned int) atoi(argv[2]);
  freq_step = atoi(argv[3]);
  step_delay_us  = (unsigned long) atoi(argv[4]);
  duration  = atoi(argv[5]);
  
  timer_dev = open(TIMER_DEV_NAME, O_RDWR);
  if (timer_dev < 0) {
    printf("%s\n", TIMER_DEV_NAME);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.%lu\n", TIMER_DEV_NAME, step_delay_us);

  int bcontinue = 1;
  timer_add.f_hz = freq_min;
  timer_add.ratio = 1;
  for (ii=0; ii<NR_PXA_CHOUT; ii++) {
    timer_add.nr = ii;
    if (ioctl(timer_dev, PXAIOCTMR_ADDCO, (void*) &timer_add)) {
      perror("ioctl PXAIOCTMR_ADDCO: ");
      bcontinue = 0;
      break;
    }
  }

  if (bcontinue) {
    struct timeval tvs, tv;
    struct pxa_timer_freq_chout freq_chout;
    
    printf("CHOUT outputs frequency changes between %u and %u Hz \
during %u seconds.\n", freq_min, freq_max, duration);
    freq_chout.f_hz  = freq_min;
    freq_chout.ratio = 1;
    gettimeofday(&tvs, NULL);

    while (bcontinue) {
      gettimeofday(&tv, NULL);
      if (tv.tv_sec > tvs.tv_sec + duration) break;
      for (ii=0; ii<NR_PXA_CHOUT; ii++) {
	freq_chout.nr = ii;
	if (ioctl(timer_dev, PXAIOCTMR_FREQCO, (void*) &freq_chout)) {
	  perror("ioctl PXAIOCTMR_FREQCO: ");
	  bcontinue = 0;
	  break;
	}
      }
      freq_chout.f_hz += freq_step;
      if (freq_chout.f_hz > freq_max) {
	freq_chout.f_hz = freq_max;
	freq_step = -freq_step;
      }
      if (freq_chout.f_hz < freq_min) {
	freq_chout.f_hz = freq_min;
	freq_step = -freq_step;
      }
      usleep(step_delay_us);
    }

    for (ii=0; ii<NR_PXA_CHOUT; ii++) {
      if (ioctl(timer_dev, PXAIOCTMR_REMCO, (void*) &ii)) {
	perror("ioctl PXAIOCTMR_REMCO: ");
	break;
      }
    }
  }

  close(timer_dev);
  exit(0);
    
 call_error:
  printf("Error. Call syntax is: test_timer2 freq_min freq_max freq_step\
 step_delay duration\n");
  printf("   freq_min is the output minimum frequency in Hertz\n");
  printf("   freq_max is the output maximum frequency in Hertz\n");
  printf("   freq_step is the frequency increment in Hertz every step_delay\n");
  printf("   step_delay is the delay in microseconds between frequency increments\n");
  printf("   duration is the test duration in seconds\n");
  exit(1);
}
