#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/pxa_gpio.h>

#define GPIO_DEV_NAME "/dev/pxa-gpio"
#define GPIO_D4       22
#define GPIO_D5       21
#define GPIO_D6       13
#define GPIO_D7        9

int nb_leds;
int leds_tab[4];


int check_args(int argc, char *argv[])
{
  int ii;

  if (argc < 2 || argc > 5)
    goto call_error;
  if (strcmp(argv[1], "-h") == 0)
    goto call_error;
  nb_leds = argc - 1;
  ii = 0;
  while (ii < nb_leds) {
    if (strcmp(argv[ii+1], "D4") == 0)
      leds_tab[ii] = GPIO_D4;
    else if (strcmp(argv[ii+1], "D5") == 0)
      leds_tab[ii] = GPIO_D5;
    else if (strcmp(argv[ii+1], "D6") == 0)
      leds_tab[ii] = GPIO_D6;
    else if (strcmp(argv[ii+1], "D7") == 0)
      leds_tab[ii] = GPIO_D7;
    else
      goto call_error;
    ii++;
  }

  return 0;

 call_error:
  printf("mbs_led [D4] [D5] [D6] [D7]\n");
  return -1;
}

int main(int argc, char *argv[])
{
  int gpio_dev, ii;
  struct pxa_gpio_add gpio_add;

  if (check_args(argc, argv) != 0)
    exit(1);
  
  gpio_dev = open(GPIO_DEV_NAME, O_RDWR);
  if (gpio_dev < 0) {
    char err_str[64];
    sprintf(err_str, "open %s", GPIO_DEV_NAME);
    perror(err_str);
    exit(1);
  }

  gpio_add.input = 0;
  gpio_add.init_state = 1;
  for (ii==0; ii<nb_leds; ii++) {
    gpio_add.gpio = leds_tab[ii];
    if (ioctl(gpio_dev, PXAIOCGPIOADD, (void*) &gpio_add))
      perror("ioctl PXAIOCGPIOADD: ");
  }
  usleep(10000);

  for (ii==0; ii<nb_leds; ii++) {
    gpio_add.gpio = leds_tab[ii];
    if (ioctl(gpio_dev, PXAIOCGPIOREM, (void*) &gpio_add.gpio))
      perror("ioctl PXAIOCGPIOREM: ");
  }

  close(gpio_dev);
  exit(0);
}
